"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isTrustworthyUrl = exports.isExternalUrl = exports.default = void 0;
var _axios = _interopRequireDefault(require("axios"));
var _qs = require("qs");
var _pagination = require("./pagination");
var _configuration = require("./configuration");
var _constants = require("./constants");
const EXTERNAL_URL_PREFIX = /^https?:\/\//;
const isExternalUrl = url => EXTERNAL_URL_PREFIX.test(url);
exports.isExternalUrl = isExternalUrl;
const isTrustworthyUrl = url => {
  const regex = (0, _configuration.getTrustedDomainRegex)();
  return regex && regex.test(url);
};
exports.isTrustworthyUrl = isTrustworthyUrl;
const paramsSerializer = params => (0, _qs.stringify)(params, {
  indices: false
});
const getNetworkError = _ref => {
  let {
    response,
    request,
    message
  } = _ref;
  if (response) {
    return {
      ...response.data,
      statusCode: response.status ? parseInt(response.status, 10) : 500
    };
  }
  if (request) {
    return {
      data: request,
      message: 'No response from server',
      statusCode: _constants.STATUS_CODE_NO_RESPONSE
    };
  }
  return {
    data: message,
    message: 'Error while creating request',
    statusCode: _constants.STATUS_CODE_REQUEST_FAILED
  };
};
const getRequestConfig = function () {
  let options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  const isExternal = isExternalUrl(options.url);
  const isTrustworthy = isTrustworthyUrl(options.url);
  const locale = options.locale || (0, _configuration.getLocaleHeader)();
  let url = options.url;
  if (!isExternal) url = `${(0, _configuration.getBaseUrl)()}${url}`;
  const config = {
    url,
    paramsSerializer,
    headers: {
      Accept: 'application/json'
    }
  };
  if (options.type) config.method = options.type;
  if (options.query) config.params = options.query;
  if (options.payload) config.data = options.payload;
  if (options.signal) {
    config.signal = options.signal;
  } else if (options.getAbortCallback) {
    // TODO: deprecate getAbortCallback in next major release
    const controller = new AbortController();
    options.getAbortCallback(() => {
      controller.abort();
    });
    config.signal = controller.signal;
  }
  if (options.type === 'query') {
    config.method = 'get';
    config.params = (0, _pagination.buildPaginationQuery)(options.query, options.pagination);
  }
  if (!isExternal || isTrustworthy) {
    if (options.token) config.headers['X-AUTH-TOKEN'] = options.token;
    if (locale) config.headers['X-Translations-Lang'] = locale;
  }
  if ((options.type === 'post' || options.type === 'put') && !options.multipart) {
    config.headers['Content-Type'] = 'application/json';
  }
  return config;
};
var _default = options => {
  const requestConfig = getRequestConfig(options);
  const requestHook = (0, _configuration.getRequestHook)();
  const responseHook = (0, _configuration.getResponseHook)();

  // These methods will mutate config object.
  options?.customize?.(requestConfig, options);
  requestHook?.(requestConfig, options);
  const pipeResponse = response => {
    const body = response && response.data ? response.data : {};
    responseHook?.(body, requestConfig, options);
    return body;
  };
  const rethrowError = error => {
    const networkError = getNetworkError(error);
    responseHook?.(networkError, requestConfig, options);
    throw networkError;
  };
  return (0, _axios.default)(requestConfig).then(pipeResponse, rethrowError);
};
exports.default = _default;